%% Example code for the paper 'Proposals and Comparisons from One-Sensor EEG and EOG Human-Machine Interfaces'
% Francisco Laport, Daniel Iglesia, Adriana Dapena, Paula M. Castro and
% Francisco J. Vazquez-Araujo.
%
% Program: Matlab
%
% This is an example of the analysis performed in the paper for the EEG
% P300 evoked potential. It reads the data of Participant 1 (P1),
% extracts the epochs, calculates the ensemble average and finally plots the
% average signal for each presented stimuli. For more information please
% refer to the aforementioned paper or the README.pdf provided with this
% example.
% 
% This example code is designed to analyze the P300 potential. For blinking
% analysis, the same procedure to read and extract the epochs should be
% performed. But the data filter and subsequent analysis should be
% different. For more information please read the aforementioned paper.
%
% For more participant data please contact Francisco Laport at
% francisco.laport@udc.es.


channel = 1;                % useful information only in channel 1 (EEG and EOG).
epoch_time = 0.6;           % in seconds. For EOG the epoch length is 1 s.
participant = 'P1';
recording_name = 'P300-1by1-R1';

% Data paths.
data_file_name = ['Data/' participant '/' participant '-' recording_name '/Raw-EEG.csv']; % EEG or EOG data.
stims_file_name = ['Data/' participant '/' participant '-' recording_name '/Raw-STIMS.csv']; % Stimuli codes.
info_file_name = ['Data/' participant '/' participant '-' recording_name '/ExpInfo.csv']; % Experiment info (target, etc.).

% Read CSV files.
data = csvread(data_file_name, 1, 1);
codes = csvread(stims_file_name, 1, 1);
info = readtable(info_file_name);

% Determine the code of target stimulus according to the paradigm.
paradigm = info{1, {'PARADIGM'}};
fs = info{1, {'FS'}};
if strcmpi(info{1, {'TARGET'}}, 'heater')
    if strcmp(paradigm, 'RC')
        stim_target = [102, 106];
    else
        stim_target = 106;
    end
else
    if strcmp(paradigm, 'RC')
        stim_target = [101, 105];
    else
        stim_target = 102;
    end
end

% Obtain trigger data for the stimuli.
dist = 0.1; % This parameter should change in EOG recording to 0.5
trigger_data = data(:, 10);
[~, peaks] = findpeaks(trigger_data, 'MinPeakDistance', dist*fs, 'MinPeakHeight', 600);

% Stimuli codes are always higher than 100. Control codes (e.g. session
% start, etc.) are always below 100.
stimuli = codes(codes(:, 1) > 100, :);

% Check if the number of detected peaks is equal to the number of stimuli.
if length(peaks) ~= size(stimuli, 1)
    error('WARNING: Peaks and stimuli does not match!');
end

% EEG/EOG data is captured in the first channel of the 8 available.
data = data(:, channel);

% Epoch length in samples.
epoch_length = epoch_time * fs;

stim_count = zeros(1, length(unique(stimuli(:,1))));
stim_sum = zeros(epoch_length, length(stim_count));

% FILTER THE DATA.
bpFilt = designfilt('bandpassiir','FilterOrder', 16, ...          
    'HalfPowerFrequency1', 1 ,'HalfPowerFrequency2', 15, ...          
    'SampleRate',fs, 'DesignMethod','butter'); 
%     fvtool(bpFilt);

data = filter(bpFilt, data);

% Extract epochs and perform ensemble average.
for peak = 1:length(peaks)
    stim = stimuli(peak, 1) - 100;
    stim_count(stim) = stim_count(stim) + 1;
    epoch = data(peaks(peak):peaks(peak)+epoch_length-1);
    stim_sum(:, stim) = stim_sum(:, stim) + (epoch - mean(epoch));
end

% Ensemble average and plot the mean signal for each stimuli.
for stim = 1:size(stim_sum,2)
    stim_avg(:, stim) =  stim_sum(:, stim)./stim_count(stim);
    
    figure(101);
    plot(stim_avg(:, stim)); hold on;
end



